<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 17/04/2015
 * Time: 10:40
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\Messages;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\DevOps\Model\Configurationmanager\Repositories as ConfigurationRepositories;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\Framework\Log\Exception;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Util\InstanceProvider;
use Zeedhi\Framework\DTO\Response\Error;

class Attachment
{

    protected $directoryBasicUpload;

    private $entityManager;
    private $fileServerUrl;
    private $httpKernel;
    /** @var Server */
    private $zhFileUploader;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Attachment */
    private $attachmentRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Project */
    private $projectRepository;
    /** @var ConfigurationRepositories\Subrelease subreleaseRepository */
    protected $subreleaseRepository;
    /** @var ConfigurationRepositories\Release $releaseRepository */
    protected $releaseRepository;
    /** @var ConfigurationRepositories\Deploy $deployRepository */
    protected $deployRepository;
    /** @var \Zeedhi\DevOps\Model\Servermanager\Repositories\ProductServer */
    protected $productServerRepository;

    public function __construct(EntityManager $entityManager, \Factory\Attachment $attachment,
                                Kernel $httpKernel, $attachmentsBaseDirectory, Server $zhFileUploader, $fileServerUrl)
    {
        $this->entityManager = $entityManager;
        $this->attachment = $attachment;
        $this->httpKernel = $httpKernel;
        $this->attachmentRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Attachment');
        $this->projectRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Project');
        $this->directoryBasicUpload = $attachmentsBaseDirectory . DIRECTORY_SEPARATOR . 'attachments';
        $this->subreleaseRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Subrelease');
        $this->releaseRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release');
        $this->deployRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Deploy');
        $this->productServerRepository = $this->entityManager->getRepository(Utilities::SERVER_MANAGER_ENTITY_DIR . 'ProductServer');
        $this->zhFileUploader = $zhFileUploader;
        $this->fileServerUrl = $fileServerUrl;
    }

    public function saveLog(array $documents, $subReleaseId, $releaseId, $deployId, $productServerId)
    {
        try {
            $filesUploaded = null;
            $subRelease = null;
            $release = null;
            $deploy = null;
            $server = null;
            if ($subReleaseId) {
                $subRelease = $this->subreleaseRepository->find($subReleaseId);
            } else if ($releaseId) {
                $release = $this->releaseRepository->find($releaseId);
            } else if ($deployId) {
                $deploy = $this->deployRepository->find($deployId);
            } else if ($productServerId) {
                $productServer = $this->productServerRepository->find($productServerId);
                $server = $productServer->getServer();
            }
            $this->updateLog($documents, $subRelease, $release, $deploy, $server);
            $this->entityManager->flush();
            return $filesUploaded;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getFullLog($labelFile)
    {
        $fileContentString = '';
        $oldLogFiles = $this->attachmentRepository->findBy(array("label" => $labelFile));
        foreach ($oldLogFiles as $oldLogFile) {
            $file = file_get_contents($this->fileServerUrl . $oldLogFile->getFileName());
            $fileContentString .= $file;
        }
        return $fileContentString;
    }

    public function getLogToSend($labelFile)
    {
        $fileContentString = '';
        $oldLogFiles = $this->attachmentRepository->findBy(array("label" => $labelFile));
        if (!empty($oldLogFiles)) {
            foreach ($oldLogFiles as $oldLogFile) {
                $file = file_get_contents($this->fileServerUrl . $oldLogFile->getFileName());
                $temp = Utilities::getTemporaryDir();
                file_put_contents($temp . DIRECTORY_SEPARATOR . $labelFile, $file);
                $fileContentString .= $file;
            }
        } else {
            $fileContentString = "";
        }

        return $fileContentString;
    }

    public function sendLogEmail($emails, $filename, $dir)
    {
        if (!empty($emails)) {
            foreach ($emails as $email) {
                InstanceProvider::getMailAttachmentMail()->sendattachMails($email, $filename, $dir);
            }
        }
    }

    public function deleteAttachment($rows)
    {
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $attachment
         */
        foreach ($rows as $row) {
            $attachment = $this->attachmentRepository->find($row['ID']);
            try {
                if ($this->verifyMultipleUses($attachment->getFileName())) {
                    $this->zhFileUploader->deleteFile($attachment->getFileName());
                }
            } catch (\Exception $e) {
                // If something goes wrong with deleteFile(), we should do nothing, only remove it from database
            }
            $this->entityManager->remove($attachment);
        }
        $this->entityManager->flush();
    }

    private function verifyMultipleUses($filePath)
    {
        $relatedAttachments = $this->attachmentRepository->findBy(array('fileName' => $filePath));
        return count($relatedAttachments) == 1;
    }

    public function deleteAttachmentByEntities(array $attachments)
    {
        /** @var Entities\Attachment[] $attachments */
        foreach ($attachments as $attachment) {
            try {
                $this->zhFileUploader->deleteFile($attachment->getFileName());
            } catch (\Exception $e) {
                // If something goes wrong with deleteFile(), we should do nothing, only remove it from database
            }
            $this->entityManager->remove($attachment);
        }
        $this->entityManager->flush();
    }

    /**
     * @param array $documents
     * @param $subRelease
     * @param $release
     * @param $deploy
     * @param $server
     * @return array
     * @throws \Exception
     */
    public function updateLog(array $documents, $subRelease, $release, $deploy, $server)
    {
        try {
            $filesUploaded = array();
            foreach ($documents as $fileData) {
                $labelFile = $fileData['name'];
                /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $oldLogFile */
                $oldLogFile = $this->attachmentRepository->findOneBy(array("label" => $labelFile));
                $typeFile = $fileData['type'];
                $sizeFile = $fileData['size'];
                if (!empty($fileData['path'])) {
                    $pathFile = $fileData['path'];
                } else {
                    throw new \Exception(Messages::ERR_LOG_FAIL_TO_SAVE);
                }
                $this->validateZhUser();
                if (is_null($oldLogFile)) {
                    $attachment = $this->attachment->createLogAttachment($typeFile, $pathFile, $labelFile, $sizeFile, $subRelease, $release, $deploy, $server);
                } else {
                    $this->zhFileUploader->deleteFile($oldLogFile->getFileName());
                    $attachment = $this->attachment->updateAttachmentPath($oldLogFile, $pathFile);
                }
                $this->entityManager->persist($attachment);
                $filesUploaded[] = $attachment;
            }
            $this->entityManager->flush();
            return $filesUploaded;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validateZhUser()
    {
        $devopsEnvironment = InstanceProvider::getDevOpsEnvironment();
        if (!$devopsEnvironment->getCurrentUserId()) {
            $cdOperador = $devopsEnvironment->getCurrentCdoperador();
            $resource = InstanceProvider::getRepository()->getResource()->findOneBy(array('externalCode' => $cdOperador));
            if($resource) {
                InstanceProvider::getSession()->set('resourceId', $resource->getId());
                $zhUser = InstanceProvider::getRepository()->getZhUser()->findOneBy(array('externalResId' => $resource->getId()));
                if($zhUser) {
                    $devopsEnvironment->setCurrentUserId($zhUser->getId());
                }
            }
        }
    }

} 